@@ -18,17 +18,11 @@
  * Default keymap, and where the kernel copy of it lives.
  */
 #ifdef __sparc__
-# define DEFMAP "sunkeymap.map"
-# define KERNDIR "/usr/src/linux/drivers/sbus/char"
+#define DEFMAP "sunkeymap.map"
+#define KERNDIR "/usr/src/linux/drivers/sbus/char"
 #else
-# define DEFMAP "defkeymap.map"
-# define KERNDIR "/usr/src/linux/drivers/tty/vt"
+#define DEFMAP "defkeymap.map"
+#define KERNDIR "/usr/src/linux/drivers/tty/vt"
 #endif
 
-#include "findfile.h"
-extern char pathname[];
-
-extern int verbose;
-extern int debug;
-
 #endif /* _PATHS_H */
